#
# gensio test utilities
#
# This file contains some classes and functions useful for testing
# gensio handling
#

import os
import gensio
import signal
import time
import curses.ascii
import sys

debug = 0

def split_version(v):
    vs = v.split(".")
    vs2 = vs[2].split("-")
    if len(vs2) > 1:
        vs = (int(vs[0]), int(vs[1]), int(vs2[0]), vs2[1])
    else:
        vs = (int(vs[0]), int(vs[1]), int(vs[2]), "")
    return vs

def gensio_version_ge(v):
    """Return True if the gensio version is >= version v, False if not"""
    try:
        gv = gensio.version
    except Exception as E:
        return True # Versions with a version macro are older than we care about

    gvs = split_version(gv)
    vs = split_version(v)
    if int(vs[0]) < int(gvs[0]):
        return True
    if int(vs[0]) > int(gvs[0]):
        return False
    if int(vs[1]) < int(gvs[1]):
        return True
    if int(vs[1]) > int(gvs[1]):
        return False
    if int(vs[2]) < int(gvs[2]):
        return True
    if int(vs[2]) > int(gvs[2]):
        return False
    return vs[3] <= gvs[3]

class Logger:
    def gensio_log(self, level, log):
        print("***%s log: %s" % (level, log))

o = gensio.alloc_gensio_selector(Logger());

class HandlerException(Exception):
    """Exception for HandleData errors"""

    def __init__(self, value):
        self.value = value
    def __repr__(self):
        return repr(self.value)
    def __str__(self):
        return str(self.value)

def buf_to_prstr(buf):
    if buf is None:
        return (0, "")
    s = ""
    for i in buf:
        if curses.ascii.isprint(i):
            s = s + chr(i)
        else:
            s = s + "\\x%2.2x" % i
    return (len(buf), s)

class HandleData:
    """Data handler for testing gensio.

    This is designed to handle input and output from gensio.  To write
    data, call set_write_data() to set some data and write it.  To wait
    for data to be read, call set_compare() to wait for the given data
    to be read.

    This just starts things up and runs asynchronously.  You can wait
    for a completion with wait() or wait_timeout().

    The io handler is in the io attribute of the object.  The handler
    object of that io will be this object.
    """

    def __init__(self, o, iostr, name = None, chunksize=10240,
                 io = None, io_is_accepter = False):
        """Start a gensio object with this handler"""
        if (name):
            self.name = name
        else:
            self.name = iostr
        self.waiter = gensio.waiter(o)
        self.to_write = None
        self.to_compare = None
        self.to_waitfor = None
        self.expecting_modemstate = False
        self.expecting_linestate = False
        self.expected_server_cb = None
        self.expected_server_value = 0
        self.expected_server_return = 0
        self.ignore_input = False
        self.expected_err = None
        self.io = None
        self.acc = None
        if io:
            if io_is_accepter:
                self.acc = io
            else:
                self.io = io
            io.set_cbs(self)
        else:
            if io_is_accepter:
                self.acc = gensio.gensio_accepter(o, iostr, self)
            else:
                self.io = gensio.gensio(o, iostr, self)

        if io_is_accepter:
            self.acc.handler = self
        else:
            self.io.handler = self
        self.chunksize = chunksize
        self.debug = 0
        return

    def set_compare(self, to_compare, start_reader = True):
        """Set some data to compare

        If start_reader is true (default), it enable the read callback.
        If the data does not compare, an exception is raised.
        """
        self.compared = 0
        if to_compare.__class__ == str:
            to_compare = bytes(to_compare, 'utf8')
        self.to_compare = to_compare
        if (start_reader):
            self.io.read_cb_enable(True)
        return

    def set_waitfor(self, waitfor, start_reader = True):
        """Wait for the given string to come in

        If start_reader is true (default), it enable the read callback.
        If the data does not compare, an exception is raised.
        """
        self.compared = 0
        self.to_waitfor = bytes(waitfor, encoding="utf8")
        if (start_reader):
            self.io.read_cb_enable(True)
        return

    def set_write_data(self, to_write, start_writer = True,
                       close_on_done = False):
        self.close_on_done = close_on_done
        self.wrpos = 0
        if to_write.__class__ == str:
            to_write = bytes(to_write, 'utf8')
        self.wrlen = len(to_write)
        self.to_write = to_write
        if (start_writer):
            self.io.write_cb_enable(True)
        return

    def set_expected_err(self, err):
        self.expected_err = err;
        return;

    def close(self):
        self.ignore_input = True
        self.io.close(self)
        return

    def shutdown(self):
        self.acc.shutdown(self)
        return

    def wait(self):
        self.waiter.wait(1)

    def wait_timeout(self, timeout):
        return self.waiter.wait_timeout(1, timeout)


    # Everything below here is internal handling functions.

    def read_callback(self, io, err, buf, auxdata):
        if self.to_compare:
            iolen = len(self.to_compare)
        elif self.to_waitfor:
            iolen = len(self.to_waitfor)
        else:
            iolen = None

        if (debug or self.debug) and buf is not None and iolen != None:
            print("%s: Got %d bytes at pos %d of %d" % (self.name, len(buf),
                                                        self.compared, iolen))
        if (debug >= 2 or self.debug >= 2):
            buflen = 0
            s = ""
            (buflen, s) = buf_to_prstr(buf)
            print("%s: Got data: (err %s %d bytes) %s" % (self.name, str(err),
                                                          buflen, s))
            if self.to_compare is not None:
                (buflen, s) = buf_to_prstr(self.to_compare[self.compared:])
                print("%s: Waiting: (%d bytes) %s" % (self.name, buflen, s))

        if err is not None and self.expected_err is not None:
            if self.expected_err != err:
                raise HandlerException(self.name + ": err: " + err)
            self.waiter.wake()
            return 0

        if (err):
            raise HandlerException(self.name + ": read: " + err)
        if (self.ignore_input):
            return len(buf)
        if (self.to_waitfor):
            for i in range(0, len(buf)):
                if buf[i] == self.to_waitfor[self.compared]:
                    self.compared += 1
                    if (len(self.to_waitfor) == self.compared):
                        self.to_waitfor = None
                        io.read_cb_enable(False)
                        self.waiter.wake()
                else:
                    self.compared = 0
            return len(buf)

        if (not self.to_compare):
            if (debug):
                print(self.name + ": Got data, but nothing to compare")
            io.read_cb_enable(False)
            return len(buf)

        if (len(buf) > len(self.to_compare)):
            count = len(self.to_compare)
        else:
            count = len(buf)

        if count > self.chunksize:
            count = self.chunksize

        for i in range(0, count):
            if (buf[i] != self.to_compare[self.compared]):
                raise HandlerException("%s: compare failure on byte %d, "
                                       "expected %x, got %x" %
                                       (self.name, self.compared,
                                        self.to_compare[self.compared],
                                        buf[i]))
            self.compared += 1

        if (self.compared >= len(self.to_compare)):
            self.to_compare = None
            io.read_cb_enable(False)
            self.waiter.wake()

        return count

    def write_callback(self, io):
        if (not self.to_write):
            if (debug or self.debug):
                print(self.name + ": Got write, but no data")
            io.write_cb_enable(False)
            return

        if (self.wrpos + self.chunksize > self.wrlen):
            wrdata = self.to_write[self.wrpos:]
        else:
            wrdata = self.to_write[self.wrpos:self.wrpos + self.chunksize]
        count = io.write(wrdata, None)
        if (debug or self.debug):
            print(self.name + ": wrote %d bytes" % count)

        if (count + self.wrpos >= self.wrlen):
            io.write_cb_enable(False)
            if (self.close_on_done):
                self.io.closeme = False
                self.close()
            self.to_write = None
            self.waiter.wake()
        else:
            self.wrpos += count
        return

    def urgent_callback(self, io):
        print(self.name + ": Urgent data")
        return

    def modemstate(self, io, modemstate):
        if (not self.expecting_modemstate):
            if (debug or self.debug):
                print("Got unexpected modemstate for %s: %x" %
                      (self.name, modemstate))
            return
        if (modemstate != self.expected_modemstate):
            raise HandlerException("%s: Expecting modemstate 0x%x, got 0x%x" %
                                   (self.name, self.expected_modemstate,
                                    modemstate))
        self.expecting_modemstate = False
        self.waiter.wake()
        return

    def set_expected_modemstate(self, modemstate):
        self.expecting_modemstate = True
        self.expected_modemstate = modemstate
        return

    def linestate(self, io, linestate):
        if (not self.expecting_linestate):
            if (debug or self.debug):
                print("Got unexpected linestate %x" % linestate)
            return
        if (linestate != self.expected_linestate):
            raise HandlerException("%s: Expecting linestate 0x%x, got 0x%x" %
                                   (self.name, self.expected_linestate,
                                    linestate))
        self.expecting_linestate = False
        self.waiter.wake()
        return

    def set_expected_linestate(self, linestate):
        self.expecting_linestate = True
        self.expected_linestate = linestate
        return

    def set_expected_server_cb(self, name, value, retval):
        self.expected_server_cb = name
        self.expected_server_value = value
        self.expected_server_return = retval
        return

    def set_expected_client_cb(self, name, value):
        self.expected_server_cb = name
        self.expected_server_value = value
        return

    def check_set_expected_telnet_cb(self, name, value):
        if not self.expected_server_cb:
            if (debug or self.debug):
                print("Got unexpected server cb: %s %d" % (name, value))
            return False
        if self.expected_server_cb != name:
            raise HandlerException(
                "Got wrong server cb, expected %s, got %s (%d)" %
                (self.expected_server_cb, name, value))
        if self.expected_server_value != value:
            raise HandlerException(
                "Got wrong server cb value for %s, expected %d, got %d" %
                (name, self.expected_server_value, value))
        self.waiter.wake()
        return True

    def baud(self, sio, err, baud):
        if not self.check_set_expected_telnet_cb("baud", baud):
            return
        return

    def datasize(self, sio, err, datasize):
        if not self.check_set_expected_telnet_cb("datasize", datasize):
            return
        return

    def parity(self, sio, err, parity):
        if not self.check_set_expected_telnet_cb("parity", parity):
            return
        return

    def stopbits(self, sio, err, stopbits):
        if not self.check_set_expected_telnet_cb("stopbits", stopbits):
            return
        return

    def flowcontrol(self, sio, err, flowcontrol):
        if not self.check_set_expected_telnet_cb("flowcontrol", flowcontrol):
            return
        return

    def iflowcontrol(self, sio, err, iflowcontrol):
        if not self.check_set_expected_telnet_cb("iflowcontrol", iflowcontrol):
            return
        return

    def sbreak(self, sio, err, sbreak):
        if not self.check_set_expected_telnet_cb("sbreak", sbreak):
            return
        return

    def dtr(self, sio, err, dtr):
        if not self.check_set_expected_telnet_cb("dtr", dtr):
            return
        return

    def rts(self, sio, err, rts):
        if not self.check_set_expected_telnet_cb("rts", rts):
            return
        return

    def sbaud(self, sio, baud):
        if not self.check_set_expected_telnet_cb("baud", baud):
            return
        sio.sg_baud(self.expected_server_return, None)
        return

    def sdatasize(self, sio, datasize):
        if not self.check_set_expected_telnet_cb("datasize", datasize):
            return
        sio.sg_datasize(self.expected_server_return, None)
        return

    def sparity(self, sio, parity):
        if not self.check_set_expected_telnet_cb("parity", parity):
            return
        sio.sg_parity(self.expected_server_return, None)
        return

    def sstopbits(self, sio, stopbits):
        if not self.check_set_expected_telnet_cb("stopbits", stopbits):
            return
        sio.sg_stopbits(self.expected_server_return, None)
        return

    def sflowcontrol(self, sio, flowcontrol):
        if not self.check_set_expected_telnet_cb("flowcontrol", flowcontrol):
            return
        sio.sg_flowcontrol(self.expected_server_return, None)
        return

    def siflowcontrol(self, sio, iflowcontrol):
        if not self.check_set_expected_telnet_cb("iflowcontrol", iflowcontrol):
            return
        sio.sg_iflowcontrol(self.expected_server_return, None)
        return

    def ssbreak(self, sio, sbreak):
        if not self.check_set_expected_telnet_cb("sbreak", sbreak):
            return
        sio.sg_sbreak(self.expected_server_return, None)
        return

    def sdtr(self, sio, dtr):
        if not self.check_set_expected_telnet_cb("dtr", dtr):
            return
        sio.sg_dtr(self.expected_server_return, None)
        return

    def srts(self, sio, rts):
        if not self.check_set_expected_telnet_cb("rts", rts):
            return
        sio.sg_rts(self.expected_server_return, None)
        return

    def close_done(self, io):
        if (debug or self.debug):
            print(self.name + ": Closed")
        self.waiter.wake()
        return

    # Accepter things below

    def log(self, acc, level, logval):
        raise HandlerException(
            "%s: Got accepter log: %s:%s:%s" % (name, acc, level, logval))
        return

    def new_connection(self, acc, io):
        if self.io is not None:
            raise HandlerException("%s: Got connection while connected" % name)
        io.handler = self
        io.is_accepter = False
        self.io = io
        io.set_cbs(self)
        if (debug or self.debug):
            print(self.name + ": New connection")
        self.waiter.wake()
        return

    def shutdown_done(self, acc):
        if (debug or self.debug):
            print(self.name + ": Shutdown")
        self.waiter.wake()
        return

try:
   import collections.abc as seqholder
except:
   import collections as seqholder

PY3 = sys.version_info[0] == 3

if PY3:
    string_types = str
else:
    string_types = basestring

def is_nonstr_sequence(obj):
    if isinstance(obj, string_types):
        return False
    return isinstance(obj, seqholder.Sequence)

class Ser2netDaemon:
    """Create a ser2net daemon instance and start it up

    ser2net is started with the given config data as a config file
    The SER2NET_EXEC environment variable can be set to tell ser2net
    to run ser2net with a specific path.

    For testing stdio handling for ser2net, you may use the io
    attribute for it but you must set its handler's ignore_input
    attribute to False or you won't get any data, and you must
    set it back to True when done.
    """

    def __init__(self, o, configdata, extra_args = ""):
        """Create a running ser2net program

        The given config data is written to a file and used as the config file.
        It is started with the -r and -d options set, you can supply extra
        options if you like as a string.
        """
        
        prog = os.getenv("SER2NET_EXEC")
        if (not prog):
            prog = "ser2net"
        configstr = ""
        if is_nonstr_sequence(configdata):
            # yaml config information
            for i in configdata:
                configstr += " -Y '" + i + "'"
        else:
            raise Exception("config data must be a sequence of strings")
        self.o = o

        args = "stdio," + prog + " -t 4 -r -d" + configstr + " " + extra_args
        if (debug):
            print("Running: " + args)
        self.handler = HandleData(o, args, name="ser2net daemon")

        self.io = self.handler.io
        self.io.closeme = True
        self.io.open_s()

        # Open stderr output
        self.err = self.io.alloc_channel(None, self)
        self.err.open_s()
        self.err.closeme = True

        self.pid = remote_id_int(self.io)
        self.handler.set_waitfor("Ready\n")
        if (self.handler.wait_timeout(2000) == 0):
            raise Exception("Timeout waiting for ser2net to start")

        self.handler.ignore_input = True

        # Uncomment the following or set it yourself to get output from
        # the ser2net daemon printed.
        #self.handler.debug = 2

        # Leave read on so if we enable debug we can see output from the
        # daemon.
        self.io.read_cb_enable(True)
        self.err.read_cb_enable(True)
        return

    def __del__(self):
        if (self.handler):
            self.terminate()
        return

    def signal(self, sig):
        """"Send a signal to ser2net"""
        os.kill(self.pid, sig)
        return

    def read_callback(self, io, err, buf, auxdata):
        if err:
            print("Error from ser2net: " + err);
            return 0

        print("Error output from ser2net: " + str(buf, 'utf8'));
        return len(buf)

    def terminate(self):
        """Terminate the running ser2net

        This closes the io and sends a SIGTERM to ser2net and waits
        a bit for it to terminate.  If it does not terminate, send
        SIGTERM a few more times.  If it still refuses to close, send
        a SIGKILL.  If all that fails, raise an exception.
        """
        if (debug):
            print("Terminating")

        count = 10
        while (count > 0):
            if (count < 6):
                self.signal(signal.SIGTERM)
            else:
                self.signal(signal.SIGKILL)
            # It would be really nice if waitpid had a timeout options,
            # in absense of that simulate it, sort of.
            subcount = 500
            while (subcount > 0):
                time.sleep(.01)
                pid, rv = os.waitpid(self.pid, os.WNOHANG)
                if (pid > 0):
                    if self.io.closeme:
                        self.handler.close()
                    if self.err.closeme:
                        self.err.close_s()
                    self.handler = None
                    return
                subcount -= 1
            count -= 1
        raise Exception("ser2net did not terminate");

def alloc_io(o, iostr, do_open = True, chunksize = 10240,
             io_is_accepter = False):
    """Allocate an io instance with a HandlerData handler

    If do_open is True (default), open it, too.
    """
    h = HandleData(o, iostr, chunksize = chunksize,
                   io_is_accepter = io_is_accepter)
    if (do_open):
        if io_is_accepter:
            h.acc.startup()
        else:
            h.io.open_s()
    if io_is_accepter:
        h.acc.is_accepter = True
        return h.acc
    else:
        h.io.is_accepter = False
        return h.io

def test_dataxfer(io1, io2, data, timeout = 1000, compare = None):
    """Test a transfer of data from io1 to io2

    If the transfer does not complete by "timeout" milliseconds, raise
    an exception.
    """
    if compare is None:
        compare = data
    io1.handler.set_write_data(data)
    io2.handler.set_compare(compare)
    if (io1.handler.wait_timeout(timeout) == 0):
        raise Exception(("%s: %s: " % ("test_dataxfer", io1.handler.name)) +

                        ("Timed out waiting for write completion at byte %d" %
                         io1.handler.wrpos))
    if (io2.handler.wait_timeout(timeout) == 0):
        raise Exception(("%s: %s: " % ("test_dataxfer", io2.handler.name)) +

                        ("Timed out waiting for read completion at byte %d" %
                         io2.handler.compared))
    return

def test_dataxfer_simul(io1, io2, data, timeout = 10000):
    """Test a simultaneous bidirectional transfer of data between io1 to io2

    If the transfer does not complete by "timeout" milliseconds, raise
    an exception.
    """
    io1.handler.set_write_data(data)
    io1.handler.set_compare(data)
    io2.handler.set_write_data(data)
    io2.handler.set_compare(data)
    if (io1.handler.wait_timeout(timeout) == 0):
        raise Exception("%s: %s: Timed out waiting for write completion" %
                        ("test_dataxfer", io1.handler.name))
    if (io2.handler.wait_timeout(timeout) == 0):
        raise Exception("%s: %s: Timed out waiting for write completion" %
                        ("test_dataxfer", io2.handler.name))
    if (io1.handler.wait_timeout(timeout) == 0):
        raise Exception("%s: %s: Timed out waiting for read completion" %
                        ("test_dataxfer", io1.handler.name))
    if (io2.handler.wait_timeout(timeout) == 0):
        raise Exception("%s: %s: Timed out waiting for read completion" %
                        ("test_dataxfer", io2.handler.name))
    return

def test_write_drain(io1, io2, data, timeout = 1000):
    """Test that a close does not loose data.

    If the transfer does not complete by "timeout" milliseconds, raise
    an exception.
    """
    io1.handler.set_write_data(data, close_on_done = True)
    io2.handler.set_compare(data)
    if (io1.handler.wait_timeout(timeout) == 0):
        raise Exception(("%s: %s: " % ("test_dataxfer", io1.handler.name)) +
                        ("Timed out waiting for write completion at byte %d" %
                         io1.handler.wrpos))
    if (io2.handler.wait_timeout(timeout) == 0):
        raise Exception(("%s: %s: " % ("test_dataxfer", io2.handler.name)) +
                        ("Timed out waiting for read completion at byte %d" %
                         io2.handler.compared))
    return

def io_close(io, timeout = 1000):
    """close the given gensio

    If it does not succeed in timeout milliseconds, raise and exception.
    """
    io.closeme = False
    if io.is_accepter:
        io.handler.shutdown()
    else:
        io.handler.close()
    if (io.handler.wait_timeout(timeout) == 0):
        raise Exception("%s: %s: Timed out waiting for close" %
                        ("io_close", io.handler.name))
    return

def setup_2_ser2net(o, config, io1str, io2str, do_io1_open = True,
                    extra_args = "", io1_is_accepter = False):
    """Setup a ser2net daemon and two gensio connections

    Create a ser2net daemon instance with the given config and two
    gensio connections with the given strings.

    If io1str is None, use the stdio of the ser2net connection as
    io1 (generally for testing stdio to ser2net).

    A "closeme" boolean attribute is added to io1 telling if They
    should be closed upon completion of the test, this is set to false
    for ser2net stdio.
    """
    io1 = None
    io2 = None
    ser2net = Ser2netDaemon(o, config, extra_args = extra_args)
    try:
        if io1str:
            io1 = alloc_io(o, io1str, do_io1_open,
                           io_is_accepter = io1_is_accepter)
            io1.closeme = do_io1_open
        else:
            io1 = ser2net.io
            io1.handler.ignore_input = False
            io1.closeme = False
        io2 = alloc_io(o, io2str)
        io2.closeme = True
    except:
        if io1:
            if io1.closeme:
                io_close(io1)
        if io2:
            io_close(io2)
        ser2net.terminate()
        raise
    return (ser2net, io1, io2)

def finish_2_ser2net(ser2net, io1, io2, handle_except = True):
    if io1.closeme:
        if io1.is_accepter and io1.handler.io is not None:
            try:
                io_close(io1.handler.io)
            except Exception as E:
                pass
        try:
            io_close(io1)
        except:
            pass
    else:
        io1.handler.ignore_input = True
    if io2.closeme:
        try:
            io_close(io2)
        except:
            pass
    else:
        io2.handler.ignore_input = True
    if handle_except and sys.exc_info()[0]:
        g = gensio.waiter(ser2net.o)
        print("Exception occurred, waiting a bit for things to clear.")
        g.wait_timeout(1, 2000)
    ser2net.terminate()
    return


keydir = os.getenv("keydir")
if not keydir:
    if (not keydir):
        keydir = "ca"

srcdir = os.getenv("srcdir")
if not srcdir:
    srcdir = os.path.dirname(sys.argv[0])
    if (not srcdir):
        srcdir = "."

def remote_id_int(io):
    return int(io.control(0, True, gensio.GENSIO_CONTROL_REMOTE_ID, None))



#!/usr/bin/python3
import gensio
import utils
from serialsim import *
import tempfile
import os

from dataxfer import test_one_xfer

def gettempfile():
    fo = tempfile.mkstemp()
    os.close(fo[0])
    return fo[1]

def validate_file_contents(test, fn, contents):
    f = open(fn)
    a = f.read()
    f.close()
    if a != contents:
        raise Exception("%s: expected contents '%s', got '%s'" %
                        (test, contents, a))

temp1 = gettempfile()
try:
    temp2 = gettempfile()
except:
    os.unlink(temp1)
    raise
try:
    temp3 = gettempfile()
except:
    os.unlink(temp1)
    os.unlink(temp2)
    raise

try:
    print("Testing file tracing")

    test_one_xfer("trace read", None, "Testing Trace Read!",
                  ("connection: &con",
                   "  accepter: tcp,3023",
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    trace-read: %s" % temp1),
                   "tcp,localhost,3023",
                   "serialdev,/dev/ttyPipeB0,9600N81")

    validate_file_contents("trace_read", temp1, "Testing Trace Read!")

    test_one_xfer("trace write", "Testing Trace Write!", None, 
                  ("connection: &con",
                   "  accepter: tcp,3023",
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    trace-write: %s" % temp1),
                   "tcp,localhost,3023",
                   "serialdev,/dev/ttyPipeB0,9600N81")

    # Notice that in appends by default.
    validate_file_contents("trace_write", temp1,
                           "Testing Trace Read!Testing Trace Write!")
    os.truncate(temp1, 0)
    os.truncate(temp2, 0)

    test_one_xfer("trace read2", None, "Testing Trace Read2!",
                  ("connection: &con",
                   "  accepter: tcp,3023",
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    trace-read: %s" % temp1,
                   "    trace-write: %s" % temp2),
                   "tcp,localhost,3023",
                   "serialdev,/dev/ttyPipeB0,9600N81")

    validate_file_contents("trace_read2", temp1, "Testing Trace Read2!")
    validate_file_contents("trace_read2/w", temp2, "")
    os.truncate(temp1, 0)

    test_one_xfer("trace write2", "Testing Trace Write2!", None,
                  ("connection: &con",
                   "  accepter: tcp,3023",
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    trace-read: %s" % temp1,
                   "    trace-write: %s" % temp2),
                   "tcp,localhost,3023",
                   "serialdev,/dev/ttyPipeB0,9600N81")

    validate_file_contents("trace_write2", temp2, "Testing Trace Write2!")
    validate_file_contents("trace_write2/r", temp1, "")
    os.truncate(temp2, 0)

    test_one_xfer("trace read/write", "Testing 2nd Trace Write!",
                  "Testing 2nd Trace Read!",
                  ("connection: &con",
                   "  accepter: tcp,3023",
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    trace-read: %s" % temp1,
                   "    trace-write: %s" % temp2),
                   "tcp,localhost,3023",
                   "serialdev,/dev/ttyPipeB0,9600N81")
    validate_file_contents("trace read/write1", temp1,
                           "Testing 2nd Trace Read!")
    validate_file_contents("trace read/write2", temp2,
                           "Testing 2nd Trace Write!")
    os.truncate(temp1, 0)
    os.truncate(temp2, 0)

    test_one_xfer("trace both", None, "Testing Trace Both!",
                  ("connection: &con",
                   "  accepter: tcp,3023",
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    trace-both: %s" % temp1),
                   "tcp,localhost,3023",
                   "serialdev,/dev/ttyPipeB0,9600N81")

    validate_file_contents("trace_both", temp1, "Testing Trace Both!")
    os.truncate(temp1, 0)

    test_one_xfer("trace both2", "Testing Trace Both2!", None,
                  ("connection: &con",
                   "  accepter: tcp,3023",
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    trace-both: %s" % temp1),
                   "tcp,localhost,3023",
                   "serialdev,/dev/ttyPipeB0,9600N81")

    validate_file_contents("trace_both", temp1, "Testing Trace Both2!")
    os.truncate(temp1, 0)

    test_one_xfer("trace all", None, "Testing Trace All!",
                  ("connection: &con",
                   "  accepter: tcp,3023",
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    trace-read: %s" % temp1,
                   "    trace-write: %s" % temp2,
                   "    trace-both: %s" % temp3),
                   "tcp,localhost,3023",
                   "serialdev,/dev/ttyPipeB0,9600N81")

    validate_file_contents("trace_both", temp1, "Testing Trace All!")
    validate_file_contents("trace_both", temp2, "")
    validate_file_contents("trace_both", temp3, "Testing Trace All!")
    os.truncate(temp1, 0)
    os.truncate(temp3, 0)

    test_one_xfer("trace all2", "Testing Trace All!", None,
                  ("connection: &con",
                   "  accepter: tcp,3023",
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    trace-read: %s" % temp1,
                   "    trace-write: %s" % temp2,
                   "    trace-both: %s" % temp3),
                   "tcp,localhost,3023",
                   "serialdev,/dev/ttyPipeB0,9600N81")

    validate_file_contents("trace_both2", temp1, "")
    validate_file_contents("trace_both2", temp2, "Testing Trace All!")
    validate_file_contents("trace_both2", temp3, "Testing Trace All!")
    os.truncate(temp1, 0)
    os.truncate(temp3, 0)

    print("  Success!")

finally:
    os.unlink(temp1)
    os.unlink(temp2)
    os.unlink(temp3)



#!/usr/bin/python3

from dataxfer import test_transfer, test_write_drain
import ipmisimdaemon
import gensio
import utils
import os

o = utils.o
isim = ipmisimdaemon.IPMISimDaemon(o)

rb = os.urandom(1048576)

test_transfer("basic ipmisol", rb,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: ipmisol,lan -U ipmiusr -P test -p %d localhost,115200" % (ipmisimdaemon.ipmisol_port)),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeA0,115200N81", timeout=150000)



#!/usr/bin/python3

import os
from dataxfer import test_transfer, test_write_drain
import ipmisimdaemon
import gensio
import utils

o = utils.o
isim = ipmisimdaemon.IPMISimDaemon(o)

rb = os.urandom(512)

test_transfer("basic ipmisol", rb,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: ipmisol,lan -U ipmiusr -P test -p %d localhost,9600" % (ipmisimdaemon.ipmisol_port)),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeA0,9600N81", timeout=5000)



#!/usr/bin/python3

from dataxfer import test_transfer, test_write_drain

test_transfer("basic stdio", "This is a test!",
              ("connection: &con",
               "  accepter: stdio",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81"),
              None,
              "serialdev,/dev/ttyPipeB0,9600N81")

test_write_drain("basic stdio", "This is a write drain test!",
                 ("connection: &con",
                  "  accepter: stdio",
                  "  connector: serialdev,/dev/ttyPipeA0,9600N81,local"),
                 None,
                 "serialdev,/dev/ttyPipeB0,9600N81")




import gensio
import utils
import time

o = utils.o

def test_transfer(name, data, config, io1str, io2str, timeout=1000,
                  extra_args=""):
    """Test a transfer between two gensio objects

    This function takes a name (for reporting), some data to transfer,
    a config file for ser2net, and transfers the data one direction,
    then the other, then both ways at the same time.
    """
    print("Transfer %s:\n  config=%s  io1=%s\n  io2=%s" %
          (name, config, io1str, io2str))

    ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str,
                                              extra_args = extra_args)
    try:
        print("  io1 to io2")
        utils.test_dataxfer(io1, io2, data, timeout=timeout)
        print("  io2 to io1")
        utils.test_dataxfer(io2, io1, data, timeout=timeout)
        print("  bidirectional")
        utils.test_dataxfer_simul(io1, io2, data, timeout=timeout)
    finally:
        utils.finish_2_ser2net(ser2net, io1, io2)
    print("  Success!")
    return

def test_write_drain(name, data, config, io1str, io2str, timeout=3000,
                     initial_write_io1 = None, switch_delay = 0.0):
    """Test that close does not lose data

    This function takes a name (for reporting), some data to transfer,
    a config file for ser2net, and writes the data and immediately
    closes the connection after the write succeeds and makes sure all
    the written data gets there.

    If initial_write_io1 is not None, the string is written to io1 when
    transferring from io2 to io1.  This is a hack for UDP, ser2net will
    not be connected to the udp port until it receives some data from
    it.
    """
    print("Write drain %s:\n  config=%s  io1=%s\n  io2=%s" %
          (name, config, io1str, io2str))

    ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str)
    try:
        print("  io1 to io2")
        utils.test_write_drain(io1, io2, data, timeout=timeout)
    finally:
        utils.finish_2_ser2net(ser2net, io1, io2)

    ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str)
    try:
        print("  io2 to io1")
        end = time.time() + switch_delay
        while (time.time() < end):
            gensio.waiter(o).wait_timeout(1, int(switch_delay / 5))
        if initial_write_io1:
            io1.handler.set_write_data(initial_write_io1)
            io2.handler.set_compare(initial_write_io1)
            if io1.handler.wait_timeout(1000) == 0:
                raise Exception(
                  "%s: Timed out on dummy write completion" % io1.handler.name)
            if io2.handler.wait_timeout(1000) == 0:
                raise Exception(
                  "%s: Timed out on dummy read completion" % io1.handler.name)
        utils.test_write_drain(io2, io1, data, timeout=timeout)
    finally:
        utils.finish_2_ser2net(ser2net, io1, io2)

    print("  Success!")
    return

def test_one_xfer(name, data1, data2, config, io1str, io2str, timeout=1000,
                  extra_args="", compare1 = None, compare2 = None):

    print("  " + name)
    ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str,
                                              extra_args = extra_args)
    try:
        if data1:
            utils.test_dataxfer(io1, io2, data1, timeout=timeout,
                                compare = compare1)
        if data2:
            utils.test_dataxfer(io2, io1, data2, timeout=timeout,
                                compare = compare2)
    finally:
        utils.finish_2_ser2net(ser2net, io1, io2)
    return

def test_connect_back(name, data, config, io1str, io2str, timeout=3000,
                      extra_args = ""):
    print("Connect back %s:\n  config=%s  io1=%s\n  io2=%s" %
          (name, config, io1str, io2str))
    ser2net, acc1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str,
                                               extra_args = extra_args,
                                               io1_is_accepter = True)
    try:
        # Send some data to start the connection.
        io2.handler.set_write_data(data)

        # Now wait for the connect back.
        if (acc1.handler.wait_timeout(timeout) == 0):
            raise Exception(
                "%s: Timed out on connect back" % acc1.handler.name)

        acc1.handler.set_compare(data)
        if (acc1.handler.wait_timeout(timeout) == 0):
            raise Exception(
                "%s: Timed out on connect back data" % acc1.handler.name)

    finally:
        utils.finish_2_ser2net(ser2net, acc1, io2)
    print("  Success!")
    return



#!/usr/bin/python3
import gensio
import utils
from serialsim import *
import tempfile
import os

print("Testing miscellaneous features")

print("  kickolduser")
ser2net, io1, io2 = utils.setup_2_ser2net(utils.o,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81",
               "  options:",
               "    kickolduser: true"),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81")
io3 = None
try:
    io1.handler.set_expected_err("Remote end closed connection")
    io1.read_cb_enable(True)
    io3 = utils.alloc_io(utils.o, "tcp,localhost,3023")
    if io1.handler.wait_timeout(1000) == 0:
        raise Exception("kickolduser: remote end didn't close")
finally:
    if io3 is not None:
        utils.io_close(io3)
    utils.finish_2_ser2net(ser2net, io1, io2)

print("  multiple connections")
ser2net, io1, io2 = utils.setup_2_ser2net(utils.o,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81",
               "  options:",
               "    max-connections: 2"),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81")
io3 = None
io4 = None
try:
    utils.test_dataxfer(io2, io1, "Test string")
    io3 = utils.alloc_io(utils.o, "tcp,localhost,3023")
    io3.handler.set_compare("Test 2 string")
    utils.test_dataxfer(io2, io1, "Test 2 string")
    if io3.handler.wait_timeout(1000) == 0:
        raise Exception("max-connections: Didn't receive data")

    io4 = utils.alloc_io(utils.o, "tcp,localhost,3023", do_open = False)
    io4.handler.set_compare("Port already in use\r\n")
    io4.handler.set_expected_err("Remote end closed connection")
    io4.open_s()
    io4.read_cb_enable(True)
    if io4.handler.wait_timeout(1000) == 0:
        raise Exception("max-connections: Overconnect didn't receive errstring")
    io4.read_cb_enable(True)
    if io4.handler.wait_timeout(1000) == 0:
        raise Exception("max-connections: Overconnect didn't remclose")

finally:
    if io3 is not None:
        utils.io_close(io3)
    if io4 is not None:
        utils.io_close(io4)
    utils.finish_2_ser2net(ser2net, io1, io2)

print("  Success!")



#!/usr/bin/python3

import os
import gensio
from dataxfer import test_transfer

rb = os.urandom(1048576)

test_transfer("sctp large random", rb,
              ("connection: &con",
               "  accepter: sctp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,115200n81"),
              "sctp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,115200N81",
              timeout=150000)



#!/usr/bin/python3

from dataxfer import test_transfer, test_write_drain, test_connect_back
import utils

test_transfer("basic ssl tcp", "This is a test!",
              ("connection: &con",
               "  accepter: ssl(key=%s/key.pem,cert=%s/cert.pem),tcp,3023"
                 % (utils.keydir, utils.keydir),
               "  connector: serialdev,/dev/ttyPipeA0,9600N81"),
              "ssl(CA=%s/CA.pem),tcp,localhost,3023" % utils.keydir,
              "serialdev,/dev/ttyPipeB0,9600N81")

test_write_drain("basic ssl tcp", "This is a write drain test!",
                 ("connection: &con",
                  "  accepter: ssl(key=%s/key.pem,cert=%s/cert.pem),tcp,3023"
                  % (utils.keydir, utils.keydir),
                  "  connector: serialdev,/dev/ttyPipeA0,9600N81,local"),
                 "ssl(CA=%s/CA.pem),tcp,localhost,3023" % utils.keydir,
                 "serialdev,/dev/ttyPipeB0,9600N81")

# Note that in the connect back we set cert=.  By default the
# certificate and private key are set by defaults in ser2net; if they
# cannot load then they will fail.  This avoids runtime failures if
# the default certificate is not present.
test_connect_back("basic ssl tcp", "SSL Connect back test!",
                  ("connection: &con",
                   "  accepter: ssl(key=%s/key.pem,cert=%s/cert.pem),tcp,3023"
                       % (utils.keydir, utils.keydir),
                   "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                   "  options:",
                   "    connback: ssl(CA=%s/CA.pem,cert=),tcp,localhost,3024"
                       % (utils.keydir)),
                 "ssl(key=%s/key.pem,cert=%s/cert.pem),tcp,localhost,3024"
                     % (utils.keydir, utils.keydir),
                 "serialdev,/dev/ttyPipeB0,9600N81")




import termios
import copy

import gensio
import utils
from serialsim import *

# This is the termios ser2net sets up when it opens a serial port.
# Same for sergensio_termios gensio.
base_termios = [ 0, 0, 0, 0, 0, 0,
                 [ '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0',
                   '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0',
                   '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0',
                   '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0' ]]

def setup_base_termios():
    base_termios[0] = termios.IGNBRK
    base_termios[2] = (termios.B9600 | termios.CREAD | termios.CS8)
    base_termios[4] = termios.B9600
    base_termios[5] = termios.B9600
    base_termios[6][termios.VTIME] = 0
    base_termios[6][termios.VMIN] = 1
    base_termios[6][termios.VSTART] = chr(17)
    base_termios[6][termios.VSTOP] = chr(19)
    return

setup_base_termios()

def dup_termios(t, iflags=0, iflags_mask=0,
                oflags=0, oflags_mask=0,
                cflags=0, cflags_mask=0,
                lflags=0, lflags_mask=0):
    """Duplicate the given termios, then apply the masks and or the values
    given."""
    n = copy.deepcopy(t)
    n[0] = (n[0] & ~iflags_mask) | iflags
    n[1] = (n[1] & ~oflags_mask) | oflags
    n[2] = (n[2] & ~cflags_mask) | cflags
    n[3] = (n[3] & ~lflags_mask) | lflags
    return n

def dup_base_termios(iflags=0, iflags_mask=0,
                     oflags=0, oflags_mask=0,
                     cflags=0, cflags_mask=0,
                     lflags=0, lflags_mask=0):
    return dup_termios(base_termios, iflags, iflags_mask, oflags, oflags_mask,
                       cflags, cflags_mask, lflags, lflags_mask)

def compare_termios(tio1, tio2):
    """Compare two termios structures.  Return -1 if they are the same,
    otherwise it returns the first byte that differs."""
    for i in range(0, 6):
        if tio1[i] != tio2[i]:
            return i;
    for i in range(0, len(tio2[6])):
        if tio1[6][i] != tio2[6][i]:
            return i + 6;
    return -1

def test_ser2net_termios(name, handler, config, io1str, io2str):
    """Test the settings of ser2net termios

    Set up a ser2net daemon and two connections, call the given handler
    which will return a termios set.  Then fetch the termios from io2
    and make sure they match.
    """
    print("termios %s:\n  config=%s  io1=%s\n  io2=%s" %
          (name, config, io1str, io2str))

    o = utils.o
    ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str)
    try:
        io1.handler.set_compare("12345")
        if (io1.handler.wait_timeout(1000) == 0):
            raise Exception("%s: %s: Timed out waiting for banner" %
                            (name, io1.handler.name))
        io1.read_cb_enable(True)
        io2.read_cb_enable(True)

        expected_termios = handler.op(io1, io2)

        io2_rem_termios = get_remote_termios(utils.remote_id_int(io2))

        c = compare_termios(expected_termios, io2_rem_termios)
        if (c != -1):
            raise Exception("Termios mismatch at %d\nExpected: %s\nBut got  %s" %
                            (c, str(expected_termios), str(io2_rem_termios)))

    finally:
        utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
    print("  Success!")



#!/usr/bin/python3

from dataxfer import test_transfer, test_write_drain
import ipmisimdaemon
import gensio
import utils

o = utils.o
isim = ipmisimdaemon.IPMISimDaemon(o)

test_transfer("basic ipmisol", "This is a test!",
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: ipmisol,lan -U ipmiusr -P test -p %d localhost,9600" % ipmisimdaemon.ipmisol_port),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeA0,9600N81")

# Give the old ipmi_sim a little time to shut down.
gensio.waiter(o).wait_timeout(1, 1000)

# Note that ipmi_sim messes with the modem state lines, so adding
# LOCAL is required on termios.  Also, we had to add a small delay
# after the opens are complete to give time for ser2net to connect to
# ipmi_sim and set things up.  Otherwise it would often lose the first
# couple of characters going from io2 to io1.
test_write_drain("basic tcp", "This is a write drain test!",
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: ipmisol,lan -U ipmiusr -P test -p %d localhost,9600" % ipmisimdaemon.ipmisol_port),
                 "tcp,localhost,3023",
                 "serialdev,/dev/ttyPipeA0,9600N81,local",
                 switch_delay = 0.25)



#!/usr/bin/python3

import os
import gensio
from dataxfer import test_transfer

rb = os.urandom(1048576)

test_transfer("udp large random", rb,
              ("connection: &con",
               "  accepter: relpkt,udp,3023",
               "  connector: serialdev,/dev/ttyPipeA1,115200n81"),
              "relpkt,udp,localhost,3023",
              "serialdev,/dev/ttyPipeB1,115200N81",
              timeout=150000)



#!/usr/bin/python3

from dataxfer import test_transfer, test_write_drain, test_connect_back

test_transfer("basic sctp", "This is a test!",
              ("connection: &con",
               "  accepter: sctp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81"),
              "sctp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81")

test_write_drain("basic sctp", "This is a write drain test!",
                 ("connection: &con",
                  "  accepter: sctp,3023",
                  "  connector: serialdev,/dev/ttyPipeA0,9600N81,local"),
                 "sctp,localhost,3023",
                 "serialdev,/dev/ttyPipeB0,9600N81")

test_connect_back("basic sctp", "SCTP Connect back test!",
                 ("connection: &con",
                  "  accepter: sctp,3023",
                  "  connector: serialdev,/dev/ttyPipeA0,9600N81",
                  "  options:",
                  "    connback: sctp,localhost,3024"),
                 "sctp,localhost,3024",
                 "serialdev,/dev/ttyPipeB0,9600N81")



#!/usr/bin/python3

import os
import gensio
from dataxfer import test_transfer
import utils

rb = os.urandom(512)

test_transfer("ssl tcp small random", rb,
              ("connection: &con",
               "  accepter: ssl(key=%s/key.pem,cert=%s/cert.pem),tcp,3023" %
                     (utils.keydir, utils.keydir),
               "  connector: serialdev,/dev/ttyPipeA0,9600n81"),
              "ssl(CA=%s/CA.pem),tcp,localhost,3023" % utils.keydir,
              "serialdev,/dev/ttyPipeB0,9600N81",
              timeout=5000)



#!/usr/bin/python3
import gensio
import utils
from serialsim import *
import tempfile
import os

from dataxfer import test_one_xfer

print("Testing string operations")

test_one_xfer("banner", None, "Testing banner!",
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81",
               "  options:",
               "    banner: \"banner1\\\\r\\\\n\""),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81",
              compare2 = "banner1\r\nTesting banner!")

print("  banner 100 times")
ser2net, io1, io2 = utils.setup_2_ser2net(utils.o,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81",
               "  options:",
               "    banner: \"banner2\\\\r\\\\n\""),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81")
try:
    errcount = 0
    for i in range(0, 100):
        io1.handler.set_compare("banner2\r\n")
        try:
            timeleft = io1.handler.wait_timeout(1000)
        except utils.HandlerException as e:
            s = str(e)
            if (s == 'tcp,localhost,3023: compare failure on byte 0, expected 62, got 44' or
                s == 'tcp,localhost,3023: compare failure on byte 0, expected 62, got 50'):
                # Got a "Device open failure" or "Port in use"
                timeleft = 1
                errcount += 1
            else:
                raise
        if timeleft == 0:
            raise Exception("banner 100 times: Didn't receive data")
        utils.io_close(io1)
        # This is not ideal, but give time for ser2net to close its side
        io1.handler.wait_timeout(50)
        io1.open_s()
        io1.handler.ignore_input = False
finally:
    utils.finish_2_ser2net(ser2net, io1, io2)
    if errcount:
        print("    non-fatal error count: %d" % errcount)

test_one_xfer("openstr", "Testing openstr!", None,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81",
               "  options:",
               "    openstr: \"str1\""),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81",
              compare1 = "str1Testing openstr!")

test_one_xfer("openstr", "Testing openstr!", None,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81",
               "  options:",
               "    openstr: \"str1\""),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81",
              compare1 = "str1Testing openstr!")

print("  closestr")
ser2net, io1, io2 = utils.setup_2_ser2net(utils.o,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81",
               "  options:",
               "    closestr: \"str2\""),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81")
try:
    utils.test_dataxfer(io1, io2, "Testing closestr")
    io2.handler.set_compare("str2")
    utils.io_close(io1)
    if io2.handler.wait_timeout(1000) == 0:
        raise Exception("closestr: Timed out waiting for closestr")
finally:
    utils.finish_2_ser2net(ser2net, io1, io2)

print("  closeon")
ser2net, io1, io2 = utils.setup_2_ser2net(utils.o,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81",
               "  options:",
               "    closeon: \"str3\""),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81")
try:
    io1.handler.set_expected_err("Remote end closed connection")
    io1.handler.set_compare("str3")
    utils.test_dataxfer(io2, io1, "str3")
    io1.read_cb_enable(True)
    if io1.handler.wait_timeout(1000) == 0:
        raise Exception("closestr: Timed out waiting for closestr")
finally:
    utils.finish_2_ser2net(ser2net, io1, io2)

print("  sendon")
ser2net, io1, io2 = utils.setup_2_ser2net(utils.o,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600N81",
               "  options:",
               "    chardelay-min: 5000000",
               "    chardelay-max: 5000000",
               "    sendon: \"\\\\r\\\\n\""),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81")
try:
    io1.handler.set_compare("test1")
    io2.handler.set_write_data("test1")
    if io1.handler.wait_timeout(250) != 0:
        raise Exception("sendon: Got data transfer incorrectly")
    io1.handler.set_compare("test1\r\n")
    io2.handler.set_write_data("\r\n")
    if io1.handler.wait_timeout(1000) == 0:
        raise Exception("closestr: Timed out waiting for sendon")
finally:
    utils.finish_2_ser2net(ser2net, io1, io2)

print("  Success!")



#!/usr/bin/python3

import os
import gensio
from dataxfer import test_transfer

rb = os.urandom(512)

test_transfer("tcp small random", rb,
              ("connection: &con",
               "  accepter: tcp,3023",
               "  connector: serialdev,/dev/ttyPipeA0,9600n81"),
              "tcp,localhost,3023",
              "serialdev,/dev/ttyPipeB0,9600N81",
              timeout=5000)



#!/usr/bin/python3
import termioschk
import termios
import utils

class basehandler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios()

termioschk.test_ser2net_termios("base termios settings", basehandler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class cs5handler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=termios.CS5,
                                           cflags_mask=termios.CSIZE)

termioschk.test_ser2net_termios("cs5 termios settings", cs5handler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n51",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class cs6handler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=termios.CS6,
                                           cflags_mask=termios.CSIZE)

termioschk.test_ser2net_termios("cs6 termios settings", cs6handler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n61",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class cs7handler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=termios.CS7,
                                           cflags_mask=termios.CSIZE)

termioschk.test_ser2net_termios("cs7 termios settings", cs7handler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n71",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class cs8handler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=termios.CS8,
                                           cflags_mask=termios.CSIZE)

termioschk.test_ser2net_termios("cs8 termios settings", cs8handler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class parevenhandler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=termios.PARENB,
                                           cflags_mask=termios.PARODD)

termioschk.test_ser2net_termios("even parity termios settings",
                                parevenhandler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600E81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class paroddhandler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=(termios.PARENB
                                                   | termios.PARODD))

termioschk.test_ser2net_termios("odd parity termios settings",
                                paroddhandler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600o81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class twostophandler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=termios.CSTOPB)

termioschk.test_ser2net_termios("2 stop bit termios settings",
                                twostophandler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n82",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class xonhandler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(iflags=(termios.IXON |
                                                   termios.IXOFF))

termioschk.test_ser2net_termios("xon/xoff termios settings",
                                xonhandler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81,xonxoff",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class rtshandler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=termios.CRTSCTS)

termioschk.test_ser2net_termios("rts/cts termios settings",
                                rtshandler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81,rtscts",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class clocalhandler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=termios.CLOCAL)

termioschk.test_ser2net_termios("clocal termios settings",
                                clocalhandler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81,local",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class hupclhandler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios(cflags=termios.HUPCL)

termioschk.test_ser2net_termios("hupcl termios settings",
                                hupclhandler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81,hangup-when-done",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class baudhandler:
    def __init__(self, speed):
        self.baud = speed;
        return

    def op(self, io1, io2):
        t = termioschk.dup_base_termios(cflags=self.baud,
                                        cflags_mask = termios.CBAUD)
        t[4] = self.baud
        t[5] = self.baud
        return t

termioschk.test_ser2net_termios("300 baud serialdev settings",
                                baudhandler(termios.B300),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,300n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("600 baud serialdev settings",
                                baudhandler(termios.B600),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,600n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("1200 baud serialdev settings",
                                baudhandler(termios.B1200),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,1200n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("2400 baud serialdev settings",
                                baudhandler(termios.B2400),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,2400n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("4800 baud serialdev settings",
                                baudhandler(termios.B4800),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,4800n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("9600 baud serialdev settings",
                                baudhandler(termios.B9600),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("19200 baud serialdev settings",
                                baudhandler(termios.B19200),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,19200n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("38400 baud serialdev settings",
                                baudhandler(termios.B38400),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,38400n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("57600 baud serialdev settings",
                                baudhandler(termios.B57600),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,57600n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("115200 baud serialdev settings",
                                baudhandler(termios.B115200),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,115200n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("230400 baud serialdev settings",
                                baudhandler(termios.B230400),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,230400n81",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

# The Python termios module doesn't support above 230400, so we can't
# test it easily here.



#!/usr/bin/python3
import termioschk
import termios
import time
import gensio
import utils
from serialsim import *

class basehandler:
    def op(self, io1, io2):
        return termioschk.dup_base_termios()

termioschk.test_ser2net_termios("base rfc2217", basehandler(),
        ("connection: &con",
         "  accepter: telnet,tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0",
         "  options:",
         "    banner: 12345"),
        "telnet,tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class cshandler:
    def __init__(self, cs, bits):
        self.cs = cs
        self.bits = bits
        return

    def op(self, io1, io2):
        sio1 = io1.cast_to_sergensio()
        bits = sio1.sg_datasize_s(self.bits)
        if (bits != self.bits):
            raise Exception("Bit value was not set, set to %d, got %d" %
                            (self.bits, bits))
        return termioschk.dup_base_termios(cflags=self.cs,
                                           cflags_mask=termios.CSIZE)

# Check that setting termios fails if rfc2217 isn't set.
goterr = False
try:
    termioschk.test_ser2net_termios("rfc2217 setting fail on rfc2217 not set",
                                    cshandler(termios.CS5, 5),
        ("connection: &con",
         "  accepter: telnet(rfc2217=false),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
          "telnet(rfc2217),tcp,localhost,3023",
          "serialdev,/dev/ttyPipeB0,9600N81")
except Exception as E:
    if str(E) != "sergensio:sg_datasize_s: Operation not supported":
        raise
    print("  Success")
    goterr = True
if not goterr:
    raise Exception("Did not get error setting telnet rfc2217 when disabled.")

# Check that setting termios fails if local rfc2217 isn't set.
goterr = False
try:
    termioschk.test_ser2net_termios("rfc2217 settings fail on no local rfc2217",
                                    cshandler(termios.CS5, 5),
            ("connection: &con",
             "  accepter: telnet(rfc2217),tcp,3023",
             "  connector: serialdev,/dev/ttyPipeA0,9600n81",
             "  options:",
             "    banner: 12345"),
            "telnet(rfc2217=0),tcp,localhost,3023",
            "serialdev,/dev/ttyPipeB0,9600N81")
except RuntimeError as E:
    if str(E) != "Error casting from gensio to sergensio":
        raise
    print("  Success")
    goterr = True
if not goterr:
    raise Exception("Did not get error setting telnet rfc2217 when disabled.")

termioschk.test_ser2net_termios("cs5 rfc2217 settings",
                                cshandler(termios.CS5, 5),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("cs6 rfc2217 settings",
                                cshandler(termios.CS6, 6),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("cs7 rfc2217 settings",
                                cshandler(termios.CS7, 7),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("cs8 rfc2217 settings",
                                cshandler(termios.CS8, 8),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class parhandler:
    def __init__(self, tval, val):
        self.tval = tval
        self.val = val
        return

    def op(self, io1, io2):
        sio1 = io1.cast_to_sergensio()
        sio1.sg_parity_s(self.val)
        return termioschk.dup_base_termios(cflags=self.tval,
                                           cflags_mask=(termios.PARODD |
                                                        termios.PARENB))

termioschk.test_ser2net_termios("even parity rfc2217 settings",
                                parhandler(termios.PARENB,
                                           gensio.SERGENSIO_PARITY_EVEN),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("odd parity rfc2217 settings",
                                parhandler(termios.PARENB | termios.PARODD,
                                           gensio.SERGENSIO_PARITY_ODD),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class twostophandler:
    def op(self, io1, io2):
        sio1 = io1.cast_to_sergensio()
        sio1.sg_stopbits_s(2)
        return termioschk.dup_base_termios(cflags=termios.CSTOPB)

termioschk.test_ser2net_termios("2 stop bit rfx2217 settings",
                                twostophandler(),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class xonhandler:
    def op(self, io1, io2):
        sio1 = io1.cast_to_sergensio()
        sio1.sg_flowcontrol_s(gensio.SERGENSIO_FLOWCONTROL_XON_XOFF)
        return termioschk.dup_base_termios(iflags=termios.IXON | termios.IXOFF)

termioschk.test_ser2net_termios("xon/xoff rfc2217 settings",
                                xonhandler(),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class rtshandler:
    def op(self, io1, io2):
        sio1 = io1.cast_to_sergensio()
        sio1.sg_flowcontrol_s(gensio.SERGENSIO_FLOWCONTROL_RTS_CTS)
        return termioschk.dup_base_termios(cflags=termios.CRTSCTS)

termioschk.test_ser2net_termios("xon/xoff rfc2217 settings",
                                xonhandler(),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

class baudhandler:
    def __init__(self, tval, val):
        self.tval = tval
        self.val = val
        return

    def op(self, io1, io2):
        sio1 = io1.cast_to_sergensio()
        sio1.sg_baud_s(self.val)
        t = termioschk.dup_base_termios(cflags=self.tval,
                                        cflags_mask=termios.CBAUD)
        t[4] = self.tval
        t[5] = self.tval
        return t

termioschk.test_ser2net_termios("300 baud rfc2217 settings",
                                baudhandler(termios.B300, 300),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("600 baud rfc2217 settings",
                                baudhandler(termios.B600, 600),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("1200 baud rfc2217 settings",
                                baudhandler(termios.B1200, 1200),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("2400 baud rfc2217 settings",
                                baudhandler(termios.B2400, 2400),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("4800 baud rfc2217 settings",
                                baudhandler(termios.B4800, 4800),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("9600 baud rfc2217 settings",
                                baudhandler(termios.B9600, 9600),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("19200 baud rfc2217 settings",
                                baudhandler(termios.B19200, 19200),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("38400 baud rfc2217 settings",
                                baudhandler(termios.B38400, 38400),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("57600 baud rfc2217 settings",
                                baudhandler(termios.B57600, 57600),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("115200 baud rfc2217 settings",
                                baudhandler(termios.B115200, 115200),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

termioschk.test_ser2net_termios("230400 baud rfc2217 settings",
                                baudhandler(termios.B230400, 230400),
        ("connection: &con",
         "  accepter: telnet(rfc2217),tcp,3023",
         "  connector: serialdev,/dev/ttyPipeA0,9600n81",
         "  options:",
         "    banner: 12345"),
        "telnet(rfc2217),tcp,localhost,3023",
        "serialdev,/dev/ttyPipeB0,9600N81")

def test_dtr():
    config = ("connection: &con",
              "  accepter: telnet(rfc2217),tcp,3023",
              "  connector: serialdev,/dev/ttyPipeA0,9600n81",
              "  options:",
              "    banner: 12345")
    io1str = "telnet(rfc2217),tcp,localhost,3023"
    io2str = "serialdev,/dev/ttyPipeB0,9600N81"

    print("serialdev dtr rfc2217:\n  config=%s  io1=%s\n  io2=%s" %
          (config, io1str, io2str))

    o = utils.o
    ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str)
    sio1 = io1.cast_to_sergensio()
    io1.handler.set_compare("12345")
    if (io1.handler.wait_timeout(1000) == 0):
        raise Exception("%s: %s: Timed out waiting for banner" %
                        ("test dtr", io1.handler.name))

    io1.read_cb_enable(True);
    io2.read_cb_enable(True);

    set_remote_null_modem(utils.remote_id_int(io2), False);

    val = sio1.sg_dtr_s(0)
    if (val != gensio.SERGENSIO_DTR_ON):
        raise Exception("Expected DTR on at start, got %d" % val);
    val = get_remote_modem_ctl(utils.remote_id_int(io2))
    if (not (val & SERIALSIM_TIOCM_DTR)):
        raise Exception("Expected remote DTR on at start");

    val = sio1.sg_dtr_s(gensio.SERGENSIO_DTR_OFF)
    if (val != gensio.SERGENSIO_DTR_OFF):
        raise Exception("Expected DTR off");
    val = get_remote_modem_ctl(utils.remote_id_int(io2))
    if (val & SERIALSIM_TIOCM_DTR):
        raise Exception("Expected remote DTR off");

    val = sio1.sg_dtr_s(gensio.SERGENSIO_DTR_ON)
    if (val != gensio.SERGENSIO_DTR_ON):
        raise Exception("Expected DTR on");
    val = get_remote_modem_ctl(utils.remote_id_int(io2))
    if (not (val & SERIALSIM_TIOCM_DTR)):
        raise Exception("Expected remote DTR on");

    set_remote_null_modem(utils.remote_id_int(io2), True);
    utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
    print("  Success!")
    return

test_dtr()

def test_rts():
    config = ("connection: &con",
              "  accepter: telnet(rfc2217),tcp,3023",
              "  connector: serialdev,/dev/ttyPipeA0,9600n81",
              "  options:",
              "    banner: 12345")
    io1str = "telnet(rfc2217),tcp,localhost,3023"
    io2str = "serialdev,/dev/ttyPipeB0,9600N81"

    print("serialdev rts rfc2217:\n  config=%s  io1=%s\n  io2=%s" %
          (config, io1str, io2str))

    o = utils.o
    ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str)
    sio1 = io1.cast_to_sergensio()
    io1.handler.set_compare("12345")
    if (io1.handler.wait_timeout(1000) == 0):
        raise Exception("%s: %s: Timed out waiting for banner" %
                        (io1.handler.name))

    io1.read_cb_enable(True);
    io2.read_cb_enable(True);

    set_remote_null_modem(utils.remote_id_int(io2), False);

    val = sio1.sg_rts_s(0)
    if (val != gensio.SERGENSIO_RTS_ON):
        raise Exception("Expected RTS on at start, got %d" % val);
    val = get_remote_modem_ctl(utils.remote_id_int(io2))
    if (not (val & SERIALSIM_TIOCM_RTS)):
        raise Exception("Expected remote RTS on at start");

    val = sio1.sg_rts_s(gensio.SERGENSIO_RTS_OFF)
    if (val != gensio.SERGENSIO_RTS_OFF):
        raise Exception("Expected RTS off");
    val = get_remote_modem_ctl(utils.remote_id_int(io2))
    if (val & SERIALSIM_TIOCM_RTS):
        raise Exception("Expected remote RTS off");

    val = sio1.sg_rts_s(gensio.SERGENSIO_RTS_ON)
    if (val != gensio.SERGENSIO_RTS_ON):
        raise Exception("Expected RTS on");
    val = get_remote_modem_ctl(utils.remote_id_int(io2))
    if (not (val & SERIALSIM_TIOCM_RTS)):
        raise Exception("Expected remote RTS on");

    set_remote_null_modem(utils.remote_id_int(io2), True);
    utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
    print("  Success!")
    return

test_rts()

def test_modemstate():
    config = ("connection: &con",
              "  accepter: telnet(rfc2217),tcp,3023",
              "  connector: serialdev,/dev/ttyPipeA0,9600n81,local")
    io1str = "telnet(rfc2217),tcp,localhost,3023"
    io2str = "serialdev,/dev/ttyPipeB0,9600N81"

    print("serialdev modemstate rfc2217:\n  config=%s  io1=%s\n  io2=%s" %
          (config, io1str, io2str))

    o = utils.o
    ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str,
                                              do_io1_open = False)
    set_remote_null_modem(utils.remote_id_int(io2), False);
    set_remote_modem_ctl(utils.remote_id_int(io2), (SERIALSIM_TIOCM_CAR |
                                           SERIALSIM_TIOCM_CTS |
                                           SERIALSIM_TIOCM_DSR |
                                           SERIALSIM_TIOCM_RNG) << 16)

    io1.handler.set_expected_modemstate(0)
    io1.open_s()
    io1.read_cb_enable(True);
    if (io1.handler.wait_timeout(2000) == 0):
        raise Exception("%s: %s: Timed out waiting for modemstate 1" %
                        ("test dtr", io1.handler.name))

    io2.read_cb_enable(True);

    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_CD_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_CD)
    set_remote_modem_ctl(utils.remote_id_int(io2), ((SERIALSIM_TIOCM_CAR << 16) |
                                           SERIALSIM_TIOCM_CAR))
    if (io1.handler.wait_timeout(2000) == 0):
        raise Exception("%s: %s: Timed out waiting for modemstate 2" %
                        ("test dtr", io1.handler.name))

    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_DSR_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_CD |
                                        gensio.SERGENSIO_MODEMSTATE_DSR)
    set_remote_modem_ctl(utils.remote_id_int(io2), ((SERIALSIM_TIOCM_DSR << 16) |
                                           SERIALSIM_TIOCM_DSR))
    if (io1.handler.wait_timeout(2000) == 0):
        raise Exception("%s: %s: Timed out waiting for modemstate 3" %
                        ("test dtr", io1.handler.name))

    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_CD |
                                        gensio.SERGENSIO_MODEMSTATE_DSR |
                                        gensio.SERGENSIO_MODEMSTATE_CTS)
    set_remote_modem_ctl(utils.remote_id_int(io2), ((SERIALSIM_TIOCM_CTS << 16) |
                                           SERIALSIM_TIOCM_CTS))
    if (io1.handler.wait_timeout(2000) == 0):
        raise Exception("%s: %s: Timed out waiting for modemstate 4" %
                        ("test dtr", io1.handler.name))

    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_RI_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_CD |
                                        gensio.SERGENSIO_MODEMSTATE_DSR |
                                        gensio.SERGENSIO_MODEMSTATE_CTS |
                                        gensio.SERGENSIO_MODEMSTATE_RI)
    set_remote_modem_ctl(utils.remote_id_int(io2), ((SERIALSIM_TIOCM_RNG << 16) |
                                           SERIALSIM_TIOCM_RNG))
    if (io1.handler.wait_timeout(2000) == 0):
        raise Exception("%s: %s: Timed out waiting for modemstate 5" %
                        ("test dtr", io1.handler.name))

    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_RI_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_CD_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_DSR_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED)
    set_remote_modem_ctl(utils.remote_id_int(io2), (SERIALSIM_TIOCM_CAR |
                                           SERIALSIM_TIOCM_CTS |
                                           SERIALSIM_TIOCM_DSR |
                                           SERIALSIM_TIOCM_RNG) << 16)
    if (io1.handler.wait_timeout(2000) == 0):
        raise Exception("%s: %s: Timed out waiting for modemstate 6" %
                        ("test dtr", io1.handler.name))

    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_CD_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_DSR_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED |
                                        gensio.SERGENSIO_MODEMSTATE_CD |
                                        gensio.SERGENSIO_MODEMSTATE_DSR |
                                        gensio.SERGENSIO_MODEMSTATE_CTS)
    set_remote_null_modem(utils.remote_id_int(io2), True);
    if (io1.handler.wait_timeout(2000) == 0):
        raise Exception("%s: %s: Timed out waiting for modemstate 7" %
                        ("test dtr", io1.handler.name))

    utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
    print("  Success!")
    return

test_modemstate()



#
# gensio test utilities
#
# This file contains some classes and functions useful for testing
# gensio handling
#

import os
import tempfile
import utils
import signal
import time

ipmisol_port = 9002

default_ipmisim_emu = """
mc_setbmc 0x20\n
\n
mc_add 0x20 0 no-device-sdrs 0x23 9 8 0x9f 0x1291 0xf02 persist_sdr\n
sel_enable 0x20 1000 0x0a\n
\n
mc_enable 0x20\n
"""

default_ipmisim_config = """
name "ser2net_sim"\n
\n
set_working_mc 0x20\n
\n
  startlan 1\n
    addr localhost %d\n
\n
    # Maximum privilege limit on the channel.\n
    priv_limit admin\n
\n
    # Allowed IPMI 1.5 authorization types\n
    allowed_auths_callback none md2 md5 straight\n
    allowed_auths_user none md2 md5 straight\n
    allowed_auths_operator none md2 md5 straight\n
    allowed_auths_admin none md2 md5 straight\n
\n
    # guid for IPMI 2.0 authorization  You can also use a string\n
    guid a123456789abcdefa123456789abcdef\n
\n
  endlan\n
\n
  sol "/dev/ttyPipeB0" 115200 nortscts\n
\n
  # Start startcmd at startup?  Default is false.\n
  startnow false\n
\n
  # Now add some users.  User 0 is invalid, user 1 is the special "anonymous"\n
  # user and cannot take a username.  Note that the users here are only\n
  # used if the persistent user config doesn't exist.\n
  #    # valid name      passw  priv-lim max-sess allowed-auths (ignored)\n
  user 1 true  ""        "test" user     10       none md2 md5 straight\n
  user 2 true  "ipmiusr" "test" admin    10       none md2 md5 straight\n
""" % (ipmisol_port)

class IPMISimDaemon:
    """Create an IPMI Sim daemon instance and start it up

    ipmi_sim is started with the given config data as a config file
    The IPMISIM_EXEC environment variable can be set to tell this code
    to run ipmi_sim with a specific path.
    """

    def __init__(self, o, configdata = None, emudata = None, extra_args = ""):
        """Create a running ipmisim program

        The given config data is written to a file and used as the config file.
        It is started with the -r and -d options set, you can supply extra
        options if you like as a string.
        """
        
        prog = os.getenv("IPMISIM_EXEC")
        if (not prog):
            prog = "ipmi_sim"

        if not configdata:
            configdata = default_ipmisim_config
        if not emudata:
            emudata = default_ipmisim_emu

        self.cfile = tempfile.NamedTemporaryFile(mode="w+")
        self.cfile.write(configdata)
        self.cfile.flush()
        self.efile = tempfile.NamedTemporaryFile(mode="w+")
        self.efile.write(emudata)
        self.efile.flush()
        self.o = o

        args = "stdio," + prog + " -p -c " + self.cfile.name + " -f " + self.efile.name + " " + extra_args
        if (utils.debug):
            print("Running: " + args)
        self.handler = utils.HandleData(o, args, name="ipmisim daemon")

        self.io = self.handler.io
        self.io.closeme = True
        self.io.open_s()

        # Uncomment the following or set it yourself to get output from
        # the ipmisim daemon printed.
        #self.handler.debug = 2

        self.pid = utils.remote_id_int(self.io)
        self.handler.set_waitfor("> ")
        if (self.handler.wait_timeout(2000) == 0):
            raise Exception("Timeout waiting for ipmi_sim to start")

        self.handler.ignore_input = True

        # Leave read on so if we enable debug we can see output from the
        # daemon.
        self.io.read_cb_enable(True)
        return

    def __del__(self):
        if (self.handler):
            self.terminate()
        return

    def signal(self, sig):
        """"Send a signal to ipmi_sim"""
        os.kill(self.pid, sig)
        return

    def terminate(self):
        """Terminate the running ipmi_sim

        This closes the io and sends a SIGTERM to ipmi_sim and waits
        a bit for it to terminate.  If it does not terminate, send
        SIGTERM a few more times.  If it still refuses to close, send
        a SIGKILL.  If all that fails, raise an exception.
        """
        if (self.handler.debug or utils.debug):
            print("Terminating")
        if self.io.closeme:
            self.handler.close()
        count = 10
        while (count > 0):
            if (count < 6):
                self.signal(signal.SIGTERM)
            else:
                self.signal(signal.SIGKILL)
            # It would be really nice if waitpid had a timeout options,
            # in absense of that simulate it, sort of.
            subcount = 500
            while (subcount > 0):
                time.sleep(.01)
                pid, rv = os.waitpid(self.pid, os.WNOHANG)
                if (pid > 0):
                    self.handler = None
                    return
                subcount -= 1
            count -= 1
        raise Exception("ipmisim did not terminate");
